use <roundedBox.scad>

width = 122;
length = 122;
height = 25;
radius = 6;
thk = 1.75;
pillarRad = 4;

difference() {
    union() {
        roundedBox(width, length, height, radius, thk);
        pillars();
        translate([0, -27, 0])
        buzzerSupport();
        translate([-11.5, -17.5, 0])
            smallFrSkyPillars();
        
        translate([-52, 20, thk])
            rotate([0, -90, 180])
                FRskyMainBoardSlots();
        
        arduinoMount();
        
        // liPo restraints
        translate([-55, -38.5, 0])
            cube([9, 2.5, height]);
        translate([-48.5, -56.5, 0])
            cube([2.5, 19.5, height]);
        translate([-40, -35.5, 0])
            cube([10, 2.5, height]);
        translate([-36.25, -35.5, 0])
            cube([2.5, 8, height]);
        translate([-12, -35.5, 0])
            cube([10, 2.5, height]);
        translate([-10, -35.5, 0])
            cube([2.5, 8, height]);
        translate([24, -56.5, 0])
            cube([2.5, 15, height]);
        translate([24, -50, 0])
            cube([8, 2.5, height]);
        translate([-48, -61, thk + radius])
            cube([75, 4.5, 2.5]);
        translate([-48, -61, height - 6])
            cube([75, 4.5, 2.5]);
    }

    switches();

    translate([0 ,40, 0])
        display();

    translate([-17.5, -27, thk + 2])
        rotate([0, 0, 90])
            trimButton();

    translate([0, 13, thk])
        encoder();

    translate([-11.5, -17.5, 0])
        FRskySmallBoard();

    translate([0, -27, -0.1])
        cylinder(d = 3, h = thk + 0.3); // buzzer sound hole

    translate([16, length / 2 - thk,  15])
        rotate([90, 0, 0])
            antenna();
        
    translate([-16, length / 2 - thk,  15])
        rotate([90, 0, 0])
            onOffSwitch();            
}
/*
color([1, 0, 0]) {
    switches();
    translate([0 ,40 , 0])
        display();
    
    translate([-52, 20, thk])
        rotate([0, -90, 180])
            FRskyMainBoard();
    
    translate([-11.5, -17.5, 0])
        FRskySmallBoard();
    
    translate([56.2, -5, 36])
        rotate([0, -90, 0])
            rotate([0, 0, 180])
                rotate([0, -20, 0])
                    arduino();
    
    translate([0, 13, thk])
        encoder();
    
    translate([-17.5, -27, thk + 2])
        rotate([0, 0, 90])
            trimButton();
            
    translate([0, -27, thk])
        cylinder(d = 12, h = 10); // buzzer
        
    translate([-45.5, -56, thk])
        liPo();
    
    translate([16, length / 2 - thk,  15])
        rotate([90, 0, 0])
            antenna();
        
    translate([-16, length / 2 - thk,  15])
        rotate([90, 0, 0])
            onOffSwitch();
}
*/
module display() {
    union() {
        translate([-15.5, -8.9, -0.1])
            cube([31, 17.8, 1.2]); // window
        translate([-18, -14.85, 1])
            cube([36, 25.5, 2.3]); // glass
        translate([-7.5, -19.1, 1])
            cube([15, 28.5, 3.2]); // ribbon
        translate([-6, 11.5, 1])
            cube([12, 3, 29]); // connector
        translate([-19, -20.75, 2])
            cube([38, 36, thk - 1]); // board + clearance
    }
}

module arduinoMount() {
    translate([41.5, -6.5, 0])
        difference() {
            cube([6, 4, thk + 5]);
            translate([0, -0.1, 0])
                rotate([0, 20, 0])
                    cube([2.6, 2.8, thk + 6]);
        }
    translate([0, -33.5, 0])
        mirror([0, 1, 0])
            translate([41.5, 10, 0])
                difference() {
                    cube([6, 4, thk + 5]);
                    translate([0, -0.1, 0])
                        rotate([0, 20, 0])
                            cube([2.6, 2.8, thk + 6]);
                }
    difference() {            
        translate([48, -27, 0])
            cube([7, 2.5, height - 2]);

        translate([35.5, -28, thk])
            rotate([0, 20, 0])
                cube([8, 4, height + 5]);
    }
}

module buzzerSupport() {
    difference() {
        cylinder(d = 16, h = thk + 5);
        translate([0, 0, thk])
            cylinder(d = 13.7, h = 10); // buzzer
    }
}

module pillars() {
    translate([-width/2+radius+pillarRad, -length/2+radius+pillarRad, 0])
        pillar(pillarRad * 2, height, 2.5, 10);
    translate([-width/2+radius+pillarRad, length/2-radius-pillarRad, 0])
        pillar(pillarRad * 2, height, 2.5, 10);
    translate([width/2-radius-pillarRad, -length/2+radius+pillarRad, 0])
        pillar(pillarRad * 2, height, 2.5, 10);
    translate([width/2-radius-pillarRad, length/2-radius-pillarRad, 0])
        pillar(pillarRad * 2, height, 2.5, 10);
}

module onOffSwitch() {
    union() {
        translate([0, 0, -8.5])
            cylinder(d = 7, h = 9.5); // threaded part
        translate([0, 0, -4])
            rotate([165, 0, 0])
                cylinder(d = 3, h = 15); // switch
        translate([-4, -6.5, 0])
            cube([8, 13, 14]); // body
    }        
}

module antenna() {
    union() {
        translate([0, 0, -9])
            cylinder(d = 7, h = 10); // threaded (through) part (long enough for 2.5 thk)
        cylinder(d = 9, h = 3.5); // inner hexagonal part
        cylinder(d = 3, h = 23); // simulated coax wire
        translate([0, 0, -110]) {
            sphere(d = 7.5);
            cylinder(d1 = 7.5, d2 = 10, h = 105); // 'rubber duck'
        }
    }
}
module liPo() { // 2-cell LiFePO4 1100mAH
    cube([69, 20, 37]);
}

module trimButton() {
    union() {
        translate([-6.5, -4.5, 0])
            cube([20, 9, 20]);
        translate([0, 0, -16]) {
            cylinder(d = 10.5, h = 7);
            cylinder(d = 8.5, h = 17);
        }
    }
}
module encoder() { // rotary encoder
    union() {
        translate([-7, -6, 0])
            cube([14, 12, 10]);
        translate([0, 0, -20])
            cylinder(d = 8.5, h=21);
    }
}

module arduino() {
    difference() {
        union() {
            cube([34, 41, 1]);
            translate([9, 4, 1])
                cube([16, 9, 100]);
        }
        translate([2.75, 3.25, -0.1])
            cylinder(d = 3, h = 1.2);
        translate([30.25, 37.75, -0.1])
            cylinder(d = 3, h = 1.2);
    }
}

module smallFrSkyPillars() {
    translate([1.95, 8.7, 0])
        difference() {
            cylinder(d = 5, h = 5);
            translate([0, 0, thk])
                cylinder(d = 2, h = 5.1 - thk);
        }

    translate([22.25, 8.7, 0])
        difference() {
            cylinder(d = 5, h = 5);
            translate([0, 0, thk])
                cylinder(d = 2, h = 5.1 - thk);
        }
}

module FRskySmallBoard() {
    difference() {
        union() {
            translate([0, 0, 5])
                cube([25.5, 16.5, 1]);
            translate([4, 1, 5])
                cube([6, 3.5, 6]);
            translate([16, 6.5, -1])
                cylinder(d = 4.2, h = 7);
            translate([7, 6.5, -1]) {
                sphere(d = 4.2);
                cylinder(d = 4.2, h = 7);
            }
            translate([7, 6.5, thk - 1])
                cylinder(d = 6, h = 7);
        }
        translate([2.75, 8.6, 4.9])
            cylinder(d = 2, h = 1.2);
        translate([22.75, 8.6, 4.9])
            cylinder(d = 2, h = 1.2);
    }
}

module FRskyMainBoard() {
    union() {
        cube([34, 54.5, 1]);
        translate([26, 46.5, 0])
            cylinder(d=9, h=5);
        translate([31, 26, 0])
            cube([4, 12, 22]);
    }
} 

module FRskyMainBoardSlots() {
    difference() {
        translate([-thk, -3, -2])
            cube([height, 4, 5.5]);

        translate([0, -1.5, -0.5])
            cube([38, 54.5, 2.2]);
    }

    difference() {
        translate([-thk, 52.5, -2])
            cube([height, 4, 5.5]);

        translate([0, 0, -0.5])
            cube([38, 54.5, 2.2]);
    }
} 

module switches() {
    translate([-35, 3, thk + .5])
        switch();
    translate([-35, 39, thk + .5])
        switch();
    translate([35, 3, thk + .5])
        rotate([0, 0, 90])
            switch();
    translate([35, 39, thk + .5])
        rotate([0, 0, 90])
            switch();
    translate([17.5, -27, thk + .5])
        rotate([0, 0, 90])
            switch();
}

module switch() {
    union() {
        translate([0, 0, -10])
            cylinder(d = 12.7, h = 11);
        cylinder(d = 18, h = 3);
        translate([-0.75, -9, 0])
            cube([1.5, 18, 23.5]);
        translate([-6.5, -17, 15])
            cube([13, 34, 8.5]);
    }
}